import json
import logging
import pickle
import sqlite3
from threading import Lock
from typing import Dict, Optional

import numpy as np

from openfl.utilities import TensorKey

logger = logging.getLogger(__name__)

__all__ = ["PersistentTensorDB"]


class PersistentTensorDB:
    """
    The PersistentTensorDB class implements a database
    for storing tensors and metadata using SQLite.

    Attributes:
        conn: The SQLite connection object.
        cursor: The SQLite cursor object.
        lock: A threading Lock object used to ensure thread-safe operations.
    """

    TENSORS_TABLE = "tensors"
    NEXT_ROUND_TENSORS_TABLE = "next_round_tensors"
    TASK_RESULT_TABLE = "task_results"
    KEY_VALUE_TABLE = "key_value_store"

    def __init__(self, db_path) -> None:
        """Initializes a new instance of the PersistentTensorDB class."""

        logger.info("Initializing persistent db at %s", db_path)
        self.conn = sqlite3.connect(db_path, check_same_thread=False)
        self.lock = Lock()

        cursor = self.conn.cursor()
        self._create_model_tensors_table(cursor, PersistentTensorDB.TENSORS_TABLE)
        self._create_model_tensors_table(cursor, PersistentTensorDB.NEXT_ROUND_TENSORS_TABLE)
        self._create_task_results_table(cursor)
        self._create_key_value_store(cursor)
        self.conn.commit()

    def _create_model_tensors_table(self, cursor, table_name) -> None:
        """Create the database table for storing tensors if it does not exist."""
        query = f"""
            CREATE TABLE IF NOT EXISTS {table_name} (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                tensor_name TEXT NOT NULL,
                origin TEXT NOT NULL,
                round INTEGER NOT NULL,
                report INTEGER NOT NULL,
                tags TEXT,
                nparray BLOB NOT NULL
            )
        """
        cursor.execute(query)

    def _create_task_results_table(self, cursor) -> None:
        """Creates a table for storing task results."""
        query = f"""
            CREATE TABLE IF NOT EXISTS {PersistentTensorDB.TASK_RESULT_TABLE} (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                collaborator_name TEXT NOT NULL,
                round_number INTEGER NOT NULL,
                task_name TEXT NOT NULL,
                data_size INTEGER NOT NULL,
                named_tensors BLOB NOT NULL
            )
        """
        cursor.execute(query)

    def _create_key_value_store(self, cursor) -> None:
        """Create a key-value store table for storing additional metadata."""
        query = f"""
            CREATE TABLE IF NOT EXISTS {PersistentTensorDB.KEY_VALUE_TABLE} (
                key TEXT PRIMARY KEY,
                value REAL NOT NULL
            )
        """
        cursor.execute(query)

    def save_task_results(
        self,
        collaborator_name: str,
        round_number: int,
        task_name: str,
        data_size: int,
        named_tensors,
    ) -> int:
        """
        Saves task results to the task_results table.

        Args:
            collaborator_name (str): Collaborator name.
            round_number (int): Round number.
            task_name (str): Task name.
            data_size (int): Data size.
            named_tensors(List): list of binary representation of tensors.
        """
        serialized_blob = pickle.dumps(named_tensors)

        # Insert into the database
        insert_query = f"""
        INSERT INTO {PersistentTensorDB.TASK_RESULT_TABLE}
        (collaborator_name, round_number, task_name, data_size, named_tensors)
        VALUES (?, ?, ?, ?, ?);
        """
        new_id = 0
        with self.lock:
            try:
                cursor = self.conn.cursor()
                cursor.execute(
                    insert_query,
                    (collaborator_name, round_number, task_name, data_size, serialized_blob),
                )
                new_id = cursor.lastrowid
                self.conn.commit()
            finally:
                cursor.close()
        return new_id

    def get_task_result_by_id(self, task_result_id: int):
        """
        Retrieve a task result by its ID.

        Args:
            task_result_id (int): The ID of the task result to retrieve.

        Returns:
            A dictionary containing the task result details, or None if not found.
        """
        with self.lock:
            cursor = self.conn.cursor()
            query = f"""
                SELECT collaborator_name, round_number, task_name, data_size, named_tensors
                FROM {PersistentTensorDB.TASK_RESULT_TABLE}
                WHERE id = ?
            """
            cursor.execute(query, (task_result_id,))
            result = cursor.fetchone()
            if result:
                collaborator_name, round_number, task_name, data_size, serialized_blob = result
                serialized_tensors = pickle.loads(serialized_blob)
                return {
                    "collaborator_name": collaborator_name,
                    "round_number": round_number,
                    "task_name": task_name,
                    "data_size": data_size,
                    "named_tensors": serialized_tensors,
                }
            return None

    def _serialize_array(self, array: np.ndarray) -> bytes:
        """Serialize a NumPy array into bytes for storing in SQLite.
        note: using pickle since in some cases the array is actually a scalar.
        """
        return pickle.dumps(array)

    def _deserialize_array(self, blob: bytes, dtype: Optional[np.dtype] = None) -> np.ndarray:
        """Deserialize bytes from SQLite into a NumPy array."""
        try:
            return pickle.loads(blob)
        except Exception as e:
            raise ValueError(f"Failed to deserialize array: {e}")

    def __repr__(self) -> str:
        """Returns a string representation of the PersistentTensorDB."""
        with self.lock:
            cursor = self.conn.cursor()
            cursor.execute("SELECT tensor_name, origin, round, report, tags FROM tensors")
            rows = cursor.fetchall()
            return f"PersistentTensorDB contents:\n{rows}"

    def finalize_round(
        self,
        tensor_key_dict: Dict[TensorKey, np.ndarray],
        next_round_tensor_key_dict: Dict[TensorKey, np.ndarray],
        round_number: int,
        best_score: float,
    ):
        """Finalize a training round by saving tensors, preparing for the next round,
        and updating metadata in the database.

        This function performs the following steps as a single transaction:
        1. Persist the tensors of the current round into the database.
        2. Persist the tensors for the next training round into the database.
        3. Reinitialize the task results table to prepare for new tasks.
        4. Update the round number and best score in the key-value store.

        If any step fails, the transaction is rolled back to ensure data integrity.

        Args:
            tensor_key_dict (Dict[TensorKey, np.ndarray]):
                A dictionary mapping tensor keys to their corresponding
                  NumPy arrays for the current round.
            next_round_tensor_key_dict (Dict[TensorKey, np.ndarray]):
                A dictionary mapping tensor keys to their corresponding
                 NumPy arrays for the next round.
            round_number (int):
                The current training round number.
            best_score (float):
                The best score achieved during the current round.

        Raises:
            RuntimeError: If an error occurs during the transaction, the transaction is rolled back,
                        and a RuntimeError is raised with the details of the failure.
        """
        with self.lock:
            try:
                # Begin transaction
                cursor = self.conn.cursor()
                cursor.execute("BEGIN TRANSACTION")
                self._persist_tensors(cursor, PersistentTensorDB.TENSORS_TABLE, tensor_key_dict)
                self._persist_next_round_tensors(cursor, next_round_tensor_key_dict)
                self._init_task_results_table(cursor)
                self._save_round_and_best_score(cursor, round_number, best_score)
                # Commit transaction
                self.conn.commit()
                logger.info(
                    f"Committed model for round {round_number}, saved {len(tensor_key_dict)}"
                    f" model tensors and {len(next_round_tensor_key_dict)}"
                    f" next round model tensors  with best_score {best_score}"
                )
            except Exception as e:
                # Rollback transaction in case of an error
                self.conn.rollback()
                raise RuntimeError(f"Failed to finalize round: {e}")

    def _persist_tensors(
        self, cursor, table_name, tensor_key_dict: Dict[TensorKey, np.ndarray]
    ) -> None:
        """Insert a dictionary of tensors into the SQLite as part of transaction"""
        for tensor_key, nparray in tensor_key_dict.items():
            tensor_name, origin, fl_round, report, tags = tensor_key
            serialized_array = self._serialize_array(nparray)
            serialized_tags = json.dumps(tags)
            query = f"""
                    INSERT INTO {table_name} (tensor_name, origin, round, report, tags, nparray)
                    VALUES (?, ?, ?, ?, ?, ?)
                """
            cursor.execute(
                query,
                (tensor_name, origin, fl_round, int(report), serialized_tags, serialized_array),
            )

    def _persist_next_round_tensors(
        self, cursor, tensor_key_dict: Dict[TensorKey, np.ndarray]
    ) -> None:
        """Persisting the last round next_round tensors."""
        drop_table_query = f"DROP TABLE IF EXISTS {PersistentTensorDB.NEXT_ROUND_TENSORS_TABLE}"
        cursor.execute(drop_table_query)
        self._create_model_tensors_table(cursor, PersistentTensorDB.NEXT_ROUND_TENSORS_TABLE)
        self._persist_tensors(cursor, PersistentTensorDB.NEXT_ROUND_TENSORS_TABLE, tensor_key_dict)

    def _init_task_results_table(self, cursor):
        """
        Creates a table for storing task results. Drops the table first if it already exists.
        """
        drop_table_query = "DROP TABLE IF EXISTS task_results"
        cursor.execute(drop_table_query)
        self._create_task_results_table(cursor)

    def _save_round_and_best_score(self, cursor, round_number: int, best_score: float) -> None:
        """Save the round number and best score as key-value pairs in the database."""
        # Create a table with key-value structure where values can be integer or float
        # Insert or update the round_number
        cursor.execute(
            """
            INSERT OR REPLACE INTO key_value_store (key, value)
            VALUES (?, ?)
        """,
            ("round_number", float(round_number)),
        )

        # Insert or update the best_score
        cursor.execute(
            """
            INSERT OR REPLACE INTO key_value_store (key, value)
            VALUES (?, ?)
        """,
            ("best_score", float(best_score)),
        )

    def get_tensors_table_name(self) -> str:
        return PersistentTensorDB.TENSORS_TABLE

    def get_next_round_tensors_table_name(self) -> str:
        return PersistentTensorDB.NEXT_ROUND_TENSORS_TABLE

    def load_tensors(self, tensor_table) -> Dict[TensorKey, np.ndarray]:
        """Load all tensors from the SQLite database and return them as a dictionary."""
        tensor_dict = {}
        with self.lock:
            cursor = self.conn.cursor()
            query = f"SELECT tensor_name, origin, round, report, tags, nparray FROM {tensor_table}"
            cursor.execute(query)
            rows = cursor.fetchall()
            for row in rows:
                tensor_name, origin, fl_round, report, tags, nparray = row
                # Deserialize the JSON string back to a Python list
                deserialized_tags = tuple(json.loads(tags))
                tensor_key = TensorKey(tensor_name, origin, fl_round, report, deserialized_tags)
                tensor_dict[tensor_key] = self._deserialize_array(nparray)
        return tensor_dict

    def get_round_and_best_score(self) -> tuple[int, float]:
        """Retrieve the round number and best score from the database."""
        with self.lock:
            cursor = self.conn.cursor()
            # Fetch the round_number
            cursor.execute(
                """
                SELECT value FROM key_value_store WHERE key = ?
            """,
                ("round_number",),
            )
            round_number = cursor.fetchone()
            if round_number is None:
                round_number = -1
            else:
                round_number = int(round_number[0])  # Cast to int

            # Fetch the best_score
            cursor.execute(
                """
                SELECT value FROM key_value_store WHERE key = ?
            """,
                ("best_score",),
            )
            best_score = cursor.fetchone()
            if best_score is None:
                best_score = 0
            else:
                best_score = float(best_score[0])  # Cast to float
        return round_number, best_score

    def clean_up(self, remove_older_than: int = 1) -> None:
        """Remove old entries from the database."""
        if remove_older_than < 0:
            return
        with self.lock:
            cursor = self.conn.cursor()
            query = f"SELECT MAX(round) FROM {PersistentTensorDB.TENSORS_TABLE}"
            cursor.execute(query)
            current_round = cursor.fetchone()[0]
            if current_round is None:
                return
            cursor.execute(
                """
                DELETE FROM tensors
                WHERE round <= ? AND report = 0
            """,
                (current_round - remove_older_than,),
            )
            self.conn.commit()

    def close(self) -> None:
        """Close the SQLite database connection."""
        self.conn.close()

    def is_task_table_empty(self) -> bool:
        """Check if the task table is empty."""
        with self.lock:
            cursor = self.conn.cursor()
            cursor.execute("SELECT COUNT(*) FROM task_results")
            count = cursor.fetchone()[0]
            return count == 0



# Copyright (C) 2020-2025 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

"""You may copy this file as the starting point of your own model."""
import jax
import keras

class CNNModel(keras.Model):
    def compute_loss_and_updates(
        self,
        trainable_variables,
        non_trainable_variables,
        x,
        y,
        training=False,
    ):
        y_pred, non_trainable_variables = self.stateless_call(
            trainable_variables,
            non_trainable_variables,
            x,
            training=training,
        )
        loss = self.compute_loss(x, y, y_pred)
        return loss, (y_pred, non_trainable_variables)

    def train_step(self, state, data):
        (
            trainable_variables,
            non_trainable_variables,
            optimizer_variables,
            metrics_variables,
        ) = state
        x, y = data

        # Get the gradient function.
        grad_fn = jax.value_and_grad(self.compute_loss_and_updates, has_aux=True)

        # Compute the gradients.
        (loss, (y_pred, non_trainable_variables)), grads = grad_fn(
            trainable_variables,
            non_trainable_variables,
            x,
            y,
            training=True,
        )

        # Update trainable variables and optimizer variables.
        (
            trainable_variables,
            optimizer_variables,
        ) = self.optimizer.stateless_apply(
            optimizer_variables, grads, trainable_variables
        )

        # Update metrics.
        new_metrics_vars = []
        logs = {}
        for metric in self.metrics:
            this_metric_vars = metrics_variables[
                len(new_metrics_vars) : len(new_metrics_vars) + len(metric.variables)
            ]
            if metric.name == "loss":
                this_metric_vars = metric.stateless_update_state(this_metric_vars, loss)
            else:
                this_metric_vars = metric.stateless_update_state(
                    this_metric_vars, y, y_pred
                )
            logs[metric.name] = metric.stateless_result(this_metric_vars)
            new_metrics_vars += this_metric_vars

        # Return metric logs and updated state variables.
        state = (
            trainable_variables,
            non_trainable_variables,
            optimizer_variables,
            new_metrics_vars,
        )
        return logs, state

    def test_step(self, state, data):
        # Unpack the data.
        x, y = data
        (
            trainable_variables,
            non_trainable_variables,
            metrics_variables,
        ) = state

        # Compute predictions and loss.
        y_pred, non_trainable_variables = self.stateless_call(
            trainable_variables,
            non_trainable_variables,
            x,
            training=False,
        )
        loss = self.compute_loss(x, y, y_pred)

        # Update metrics.
        new_metrics_vars = []
        logs = {}
        for metric in self.metrics:
            this_metric_vars = metrics_variables[
                len(new_metrics_vars) : len(new_metrics_vars) + len(metric.variables)
            ]
            if metric.name == "loss":
                this_metric_vars = metric.stateless_update_state(this_metric_vars, loss)
            else:
                this_metric_vars = metric.stateless_update_state(
                    this_metric_vars, y, y_pred
                )
            logs[metric.name] = metric.stateless_result(this_metric_vars)
            new_metrics_vars += this_metric_vars

        # Return metric logs and updated state variables.
        state = (
            trainable_variables,
            non_trainable_variables,
            new_metrics_vars,
        )
        return logs, state



# Copyright 2020-2024 Intel Corporation
# SPDX-License-Identifier: Apache-2.0


from openfl.interface.aggregation_functions.experimental.privileged_aggregation import (
    PrivilegedAggregationFunction,
)



# Copyright 2020-2024 Intel Corporation
# SPDX-License-Identifier: Apache-2.0


"""Experimental CLI."""

from logging import getLogger
from pathlib import Path
from subprocess import check_call
from sys import executable

from click import echo, group, pass_context

import openfl

logger = getLogger(__name__)


@group()
@pass_context
def experimental(context):
    """Manage experimental capabilities in OpenFL.

    Currently, this command is used to activate Workflow API in OpenFL.
    """
    context.obj["group"] = "experimental"


@experimental.command(name="activate")
def activate():
    """
    Activates experimental environment.
    This command takes no additional arguments.
    """
    settings = Path("~").expanduser().joinpath(".openfl").resolve()
    settings.mkdir(parents=False, exist_ok=True)
    settings = settings.joinpath("experimental").resolve()

    rf = (
        Path(openfl.__file__)
        .parent.parent.resolve()
        .joinpath(
            "openfl-tutorials",
            "experimental",
            "workflow",
            "workflow_interface_requirements.txt",
        )
        .resolve()
    )

    if rf.is_file():
        check_call([executable, "-m", "pip", "install", "-r", rf], shell=False)
    else:
        logger.warning(f"Requirements file {rf} not found.")

    with open(settings, "w") as f:
        f.write("experimental")


@experimental.command(name="deactivate")
def deactivate():
    """
    Deactivates experimental environment.
    This command takes no additional arguments.
    """
    settings = Path("~").expanduser().joinpath(".openfl", "experimental").resolve()
    if settings.is_file():
        settings.unlink()
        echo("Experimental capabilities deactivated.")
    else:
        echo("Experimental capabilities are not active, no changes were made.")



# Copyright (C) 2024 Intel Corporation
# Licensed subject to the terms of the separately executed evaluation license agreement between
# Intel Corporation and you.

import numpy as np

from typing import Iterator, Tuple

from openfl.federated import PyTorchTaskRunner
from openfl.utilities import Metric


class TemplateTaskRunner(PyTorchTaskRunner):
    """Template Task Runner for PyTorch.

    This class should be used as a template to create a custom Task Runner for your specific model
    and training workload.
    After generating this template, you should:
    1. Define your model, optimizer, and loss function as you would in PyTorch.
    PyTorchTaskRunner inherits from torch.nn.Module.
    2. Implement the `train_` and `validate_` functions to define a single train
    and validate epoch of your workload.
    3. Modify the `plan.yaml` file to use this Task Runner.

    The `plan.yaml` modifications should be done under the `<workspace>/plan/plan.yaml` section:
    ```
    task_runner:
        defaults : plan/defaults/task_runner.yaml
        template: src.taskrunner.TemplateTaskRunner # Modify this line appropriately
        if you change the class name
        settings:
            # Add additional arguments that you wish to pass through `__init__`
    ```

    Define the `forward` method of this class as a forward pass through the model.
    """

    def __init__(self, device="cpu", **kwargs):
        """Initialize the Task Runner.

        Args:
            device: The hardware device to use for training (Default = "cpu").
            **kwargs: Additional arguments that may be defined in `plan.yaml`
        """
        super().__init__(device=device, **kwargs)

        # Define the model
        self.model = None  # Replace with your model
        self.to(device)

        # Define the optimizer
        self.optimizer = None  # Replace with your optimizer

        # Define the loss function
        self.loss_fn = None  # Replace with your loss function

    def forward(self, x):
        """Forward pass of the model.

        Args:
            x: Data input to the model for the forward pass.

        Returns:
            The output of the model's forward pass.
        """
        return self.model(x)

    def train_(
        self, train_dataloader: Iterator[Tuple[np.ndarray, np.ndarray]]
    ) -> Metric:
        """Single Training epoch.

        Args:
            train_dataloader: Train dataset batch generator. Yields (samples, targets) tuples
                              of size = `self.train_dataloader.batch_size`.

        Returns:
            Metric: An object containing the name of the metric and its value as an np.ndarray.
        """
        # Implement training logic here and return a Metric object with the training loss.
        # Replace the following placeholder with actual training code.
        for data, target in train_dataloader:
            # Training code
            pass

        loss = None  # Placeholder for loss calculation.
        return Metric(name="training loss", value=np.array(loss))

    def validate_(
        self, validation_dataloader: Iterator[Tuple[np.ndarray, np.ndarray]]
    ) -> Metric:
        """Single validation epoch.

        Args:
            validation_dataloader: Validation dataset batch generator. Yields (samples, targets)
                                   tuples of size = `self.validation_dataloader.batch_size`.

        Returns:
            Metric: An object containing the name of the metric and its value as an np.ndarray.
        """
        # Implement validation logic here and return a Metric object with the validation accuracy.
        # Replace the following placeholder with actual validation code.

        for data, target in validation_dataloader:
            # validation code
            pass

        accuracy = None  # Placeholder for accuracy calculation.
        return Metric(name="accuracy", value=np.array(accuracy))


raise NotImplementedError(
    "Use <workspace>/src/taskrunner.py template to create a custom Task Runner "
    "with your model definition and training/validation logic. Then remove this line."
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: openfl/protocols/aggregator.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from openfl.protocols import base_pb2 as openfl_dot_protocols_dot_base__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!openfl/protocols/aggregator.proto\x12\x11openfl.aggregator\x1a\x1bopenfl/protocols/base.proto\"o\n\rMessageHeader\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x10\n\x08receiver\x18\x02 \x01(\t\x12\x17\n\x0f\x66\x65\x64\x65ration_uuid\x18\x03 \x01(\t\x12#\n\x1bsingle_col_cert_common_name\x18\x04 \x01(\t\"?\n\x0bPingRequest\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\"@\n\x0cPingResponse\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\"C\n\x0fGetTasksRequest\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\"S\n\x04Task\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rfunction_name\x18\x02 \x01(\t\x12\x11\n\ttask_type\x18\x03 \x01(\t\x12\x13\n\x0b\x61pply_local\x18\x04 \x01(\x08\"\xa4\x01\n\x10GetTasksResponse\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\x12\x14\n\x0cround_number\x18\x02 \x01(\x05\x12&\n\x05tasks\x18\x03 \x03(\x0b\x32\x17.openfl.aggregator.Task\x12\x12\n\nsleep_time\x18\x04 \x01(\x05\x12\x0c\n\x04quit\x18\x05 \x01(\x08\"\xb1\x01\n\x1aGetAggregatedTensorRequest\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\x12\x13\n\x0btensor_name\x18\x02 \x01(\t\x12\x14\n\x0cround_number\x18\x03 \x01(\x05\x12\x0e\n\x06report\x18\x04 \x01(\x08\x12\x0c\n\x04tags\x18\x05 \x03(\t\x12\x18\n\x10require_lossless\x18\x06 \x01(\x08\"\x83\x01\n\x1bGetAggregatedTensorResponse\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\x12\x14\n\x0cround_number\x18\x02 \x01(\x05\x12\x1c\n\x06tensor\x18\x03 \x01(\x0b\x32\x0c.NamedTensor\"\x9a\x01\n\x0bTaskResults\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\x12\x14\n\x0cround_number\x18\x02 \x01(\x05\x12\x11\n\ttask_name\x18\x03 \x01(\t\x12\x11\n\tdata_size\x18\x04 \x01(\x05\x12\x1d\n\x07tensors\x18\x05 \x03(\x0b\x32\x0c.NamedTensor\"P\n\x1cSendLocalTaskResultsResponse\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\"\xd4\x01\n\x0eInteropMessage\x12\x30\n\x06header\x18\x01 \x01(\x0b\x32 .openfl.aggregator.MessageHeader\x12\x1c\n\x07message\x18\x02 \x01(\x0b\x32\x0b.DataStream\x12\x41\n\x08metadata\x18\x03 \x03(\x0b\x32/.openfl.aggregator.InteropMessage.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32\xd8\x03\n\nAggregator\x12I\n\x04Ping\x12\x1e.openfl.aggregator.PingRequest\x1a\x1f.openfl.aggregator.PingResponse\"\x00\x12U\n\x08GetTasks\x12\".openfl.aggregator.GetTasksRequest\x1a#.openfl.aggregator.GetTasksResponse\"\x00\x12v\n\x13GetAggregatedTensor\x12-.openfl.aggregator.GetAggregatedTensorRequest\x1a..openfl.aggregator.GetAggregatedTensorResponse\"\x00\x12X\n\x14SendLocalTaskResults\x12\x0b.DataStream\x1a/.openfl.aggregator.SendLocalTaskResultsResponse\"\x00(\x01\x12V\n\x0cInteropRelay\x12!.openfl.aggregator.InteropMessage\x1a!.openfl.aggregator.InteropMessage\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'openfl.protocols.aggregator_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_INTEROPMESSAGE_METADATAENTRY']._loaded_options = None
  _globals['_INTEROPMESSAGE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGEHEADER']._serialized_start=85
  _globals['_MESSAGEHEADER']._serialized_end=196
  _globals['_PINGREQUEST']._serialized_start=198
  _globals['_PINGREQUEST']._serialized_end=261
  _globals['_PINGRESPONSE']._serialized_start=263
  _globals['_PINGRESPONSE']._serialized_end=327
  _globals['_GETTASKSREQUEST']._serialized_start=329
  _globals['_GETTASKSREQUEST']._serialized_end=396
  _globals['_TASK']._serialized_start=398
  _globals['_TASK']._serialized_end=481
  _globals['_GETTASKSRESPONSE']._serialized_start=484
  _globals['_GETTASKSRESPONSE']._serialized_end=648
  _globals['_GETAGGREGATEDTENSORREQUEST']._serialized_start=651
  _globals['_GETAGGREGATEDTENSORREQUEST']._serialized_end=828
  _globals['_GETAGGREGATEDTENSORRESPONSE']._serialized_start=831
  _globals['_GETAGGREGATEDTENSORRESPONSE']._serialized_end=962
  _globals['_TASKRESULTS']._serialized_start=965
  _globals['_TASKRESULTS']._serialized_end=1119
  _globals['_SENDLOCALTASKRESULTSRESPONSE']._serialized_start=1121
  _globals['_SENDLOCALTASKRESULTSRESPONSE']._serialized_end=1201
  _globals['_INTEROPMESSAGE']._serialized_start=1204
  _globals['_INTEROPMESSAGE']._serialized_end=1416
  _globals['_INTEROPMESSAGE_METADATAENTRY']._serialized_start=1369
  _globals['_INTEROPMESSAGE_METADATAENTRY']._serialized_end=1416
  _globals['_AGGREGATOR']._serialized_start=1419
  _globals['_AGGREGATOR']._serialized_end=1891
# @@protoc_insertion_point(module_scope)
